@@ -41,6 +41,7 @@ sub new ($%) {
 		aid           => delete $options{aid} || 0,
 		credits       => delete $options{credits} || 0,
 		credit_charge => delete $options{credit_charge} || ($options{code} ? 1 : 0),
+		chain_offset  => delete $options{chain_offset} || 0,
 		struct_size   => delete $options{struct_size} || 2,
 		%options,
 	);
@@ -61,4 +62,11 @@ sub is_signed ($) {
 		($self->flags & FLAGS_SIGNED) != 0;
 }
 
+sub is_chained ($) {
+	my $self = shift;
+
+	return
+		($self->flags & FLAGS_CHAINED) != 0;
+}
+
 1;
